import Foundation

/// Helper that creates a TTLBackendConnectionSetup from user input. The TTLBackendConnectionSetup is used to
/// establish a backend connection using the SDKs TTLServiceClient.
extension TTLBackendConnectionSetup {
    static func create(credentials:Credentials, device: String, backend: BackendType) -> TTLBackendConnectionSetup {
        let backendConfig = BackendReslover.sharedInstance.getBackendConfig(type: backend)
        
        let backendConnectionSetup = TTLBackendConnectionSetup()
        backendConnectionSetup.wfcBaseUrl = backendConfig.connectDomain
        backendConnectionSetup.apiKey = backendConfig.connectApiKey
        backendConnectionSetup.wfQprsQueueHost = backendConfig.gprsQueueDomain
        backendConnectionSetup.wfQprsQueuePort = UInt16(backendConfig.gprsQueuePort)
        backendConnectionSetup.deviceId = device
        backendConnectionSetup.userName = credentials.account
        backendConnectionSetup.password = credentials.password
        
        return backendConnectionSetup
    }
}
