import os.log

#if DEBUG

/**
 * Creates a OSLog with information about the logging place.
 *
 * Under DEBUG this function creates a OSLog where
 * - subsystem is equal to Bundle.main.bundleIdentifier or ""
 * - category is a strings composed of file:function:line
 *
 * Under none DEBUG this function returns just OSLog.default
 *
 * Usage as part of a os_log call
 * - os_log("foo %@", log: createTraceLog(), Foo())
 */
public func createTraceLog(includeFunction: Bool = false, function: String = #function, file: String = #file, line: Int = #line) -> OSLog {
    // file name without path and .swift suffix (it is equal to the class name in most cases)
    let fileName = URL(fileURLWithPath: file).lastPathComponent.replacingOccurrences(of: ".swift", with: "")
    let lineNumber = String(format: "%03d", line)
    if includeFunction {
        var functionName = function
        if let i = function.firstIndex(of: "(") {
            // remove arguments
            functionName = String(function.prefix(upTo: i))
        }
        return OSLog(subsystem: Bundle.main.bundleIdentifier ?? "", category: "\(fileName):\(lineNumber):\(functionName)")
    } else {
        return OSLog(subsystem: Bundle.main.bundleIdentifier ?? "", category: "\(fileName):\(lineNumber)")
    }
}

#else

public func createTraceLog(includeFunction: Bool = false, function: String = #function, file: String = #file, line: Int = #line) -> OSLog {
    return OSLog.default
}

#endif

// Convenience function for the case that just a trace (without function name) and a optional string value should be logged
public func os_log_trace(function: String = #function, file: String = #file, line: Int = #line, _ value: String = "") {
    os_log("%{private}@", log: createTraceLog(includeFunction: false, function: function, file: file, line: line), value)
}

// Convenience function for the case that just a trace (including function name) and a optional message (without arguments) should be logged
public func os_log_trace_f(function: String = #function, file: String = #file, line: Int = #line, _ value: String = "") {
    os_log("%{private}@", log: createTraceLog(includeFunction: true, function: function, file: file, line: line), value)
}
