import UIKit

extension UITextField {
   
    /// Shakes the textfield and turns it red for an instant to signal an user input error.
    func notifyError() {
        let backgroundColor = self.backgroundColor
        UIView.animate(withDuration: 0.4, delay: 0, options: [], animations: {
            self.backgroundColor = UIColor.red
        }, completion: { _ in
            self.backgroundColor = backgroundColor
        })
        
        let shake: CABasicAnimation = CABasicAnimation(keyPath: "position")
        shake.duration = 0.07
        shake.repeatCount = 3
        shake.autoreverses = true
        shake.fromValue = NSValue(cgPoint: CGPoint(x: self.center.x - 10, y: self.center.y))
        shake.toValue = NSValue(cgPoint: CGPoint(x: self.center.x + 10, y: self.center.y))
        self.layer.add(shake, forKey: "position")
    }
    
    /**
     Provides a simple validation mechanism.
     - Returns: the validated text of the text field
     - Throws: `ValidatorError` in case of a validation issue
     */
    func validatedText(validationType: ValidatorType) throws -> String {
        let validator = ValidatorFactory.validatorFor(type: validationType)
        return try validator.validate(self)
    }
}
