import Foundation

/// Erros that can occure during the activation process.
enum ActivationError: String, CustomDebugStringConvertible
{
    /// The provided credentials were incorrect.
    case wrongCredentials
    /// The provided device can't be used in combination with the provided credentials,
    /// likely because the device belongs to another account.
    case wrongDevice
    /// Device connection failed during activation.
    case deviceDisconnected
    /// Backend connection failed during activation.
    case backendDisconnected
    /// Activation failed due to another reason.
    case activationFailed
    
    var debugDescription: String {
        return self.rawValue
    }
}
