import Foundation

enum LoginActivationPresenterState : Equatable, CustomDebugStringConvertible {
    /// Start state, waiting for activation to be triggered.
    case start
    /// Connecting to backend, waiting for backend connection to be established.
    case connectingToBackend
    /// Activating device, waiting for activation to complete.
    case activating
    /// Activation finished successfully if `error` was `nil`, otherwise activation failed.
    case finished(error:ActivationError?)
    
    private var value: Int {
        switch self {
        case .start: return 0
        case .connectingToBackend: return 1
        case .activating: return 2
        case .finished: return 3
        }
    }
    
    static func ==(lhs: LoginActivationPresenterState, rhs: LoginActivationPresenterState) -> Bool {
        // ignore concrete error value
        return lhs.value == rhs.value
    }
    
    var debugDescription: String {
        switch self {
        case .start: return "start"
        case .connectingToBackend: return "connectingToBackend"
        case .activating: return "activating"
        case .finished(let error): return "finished (error: " + String(reflecting: error) + ")"
        }
    }
}
