import Foundation

/// Presenter handling the list of available devices that can be selected during login.
class LoginDevicePresenter : NSObject, ConnectionServiceDelegate{
    private static let NONE_DEVICE = "None"
    private let connectionService : ConnectionService
    private var selectedDevice = NONE_DEVICE
    private(set) var availableDevices : [String]
    weak var delegate : LoginDevicePresenterDelegate?
    
    init(connectionService: ConnectionService) {
        self.connectionService = connectionService
        self.availableDevices = [LoginDevicePresenter.NONE_DEVICE] + connectionService.devices.sorted()
        super.init()
        self.connectionService.addDelegate(delegate: self)
    }
    
    func selectDevice(_ index: Int) {
        guard(index != 0 && index < availableDevices.count) else
        {
            selectedDevice = LoginDevicePresenter.NONE_DEVICE
            return
        }
        selectedDevice = availableDevices[index]
    }

    func deviceDisplayText() -> String {
        guard(selectedDevice != LoginDevicePresenter.NONE_DEVICE) else
        {
            return ""
        }
        return selectedDevice
    }
    
    func onDeviceListChanged(devices: Set<String>) {
        availableDevices = [LoginDevicePresenter.NONE_DEVICE] + devices.sorted()
        if selectedDevice != LoginDevicePresenter.NONE_DEVICE && !devices.contains(selectedDevice)
        {
            selectedDevice = LoginDevicePresenter.NONE_DEVICE
        }
        delegate?.onDeviceListChanged();
    }
}

protocol LoginDevicePresenterDelegate : class {
    func onDeviceListChanged()
}
