import Foundation

/**
 Presenter which is used to display device and server connection status on the main screen.
 */
class DeviceStatusPresenter: ConnectionServiceDelegate {
    private let connectionService: ConnectionService
    private let userService: UserService
    weak var delegate : DeviceStatusPresenterDelegate?
    
    init(connectionService: ConnectionService, userService: UserService) {
        self.connectionService = connectionService
        self.userService = userService
        self.connectionService.addDelegate(delegate: self)
    }
    
    var deviceConnected: Bool {
        return connectionService.connected(to: userService.deviceIdentifier)
    }
    
    var serverConnected: Bool {
        return connectionService.serverConnected(for: userService.deviceIdentifier)
    }
    
    var deviceConnectionStatus: String {
        return deviceConnected ? localize(key: "DeviceStatusPresenter.connected", withArgs: userService.deviceIdentifier) : localize(key: "DeviceStatusPresenter.disconnected", withArgs: userService.deviceIdentifier)
    }
    
    var serverConnectionStatus: String {
        return serverConnected ? localize(key: "DeviceStatusPresenter.serverConnected") : localize(key: "DeviceStatusPresenter.serverDisconnected")
    }
    
    func onServerConnected(deviceIdentifier: String) {
        guard userService.deviceIdentifier == deviceIdentifier else { return }
        delegate?.onConnectionStateChanged()
    }
    
    func onServerDisconnected(deviceIdentifier: String, reason: ServerDisconnectReason) {
        guard userService.deviceIdentifier == deviceIdentifier else { return }
        delegate?.onConnectionStateChanged()
    }
    
    func onDeviceConnected(deviceIdentifier: String) {
        guard userService.deviceIdentifier == deviceIdentifier else { return }
        delegate?.onConnectionStateChanged()
    }
    
    func onDeviceDisconnected(deviceIdentifier: String) {
        guard userService.deviceIdentifier == deviceIdentifier else { return }
        delegate?.onConnectionStateChanged()
    }
}

protocol DeviceStatusPresenterDelegate: class {
    func onConnectionStateChanged()
}
