//
// Created by Alexander Rupsch on 26/09/15.
// Copyright (c) 2015 TomTom Telematics. All rights reserved.
//

#import <Foundation/Foundation.h>


// TODO needs new obd chunks without acc (could use the old one and deprecate the acc values).
// TODO new acc chunk

@protocol TTLOBDDatagram <NSObject, NSCopying, NSMutableCopying>
/** Virtual odometer */
@property(nonatomic,  readonly) NSNumber * _Nullable odometer;
/** Fuel system status */
@property(nonatomic,  readonly) NSNumber * _Nullable fuelSystemStatus;
/** Calculated engine load */
@property(nonatomic,  readonly) NSNumber * _Nullable engineLoad;
/** Engine coolant temperature */
@property(nonatomic,  readonly) NSNumber * _Nullable coolantTemperature;
/** Intake manifold pressure */
@property(nonatomic,  readonly) NSNumber * _Nullable manifoldPressure;
/** Engine RPM */
@property(nonatomic,  readonly) NSNumber * _Nullable engineRpm;
/** Vehicle speed */
@property(nonatomic,  readonly) NSNumber * _Nullable vehicleSpeed;
/** Intake air temperature */
@property(nonatomic,  readonly) NSNumber * _Nullable airTemperature;
/** Throttle position */
@property(nonatomic,  readonly) NSNumber * _Nullable throttlePosition;
/** Mass air flow */
@property(nonatomic,  readonly) NSNumber * _Nullable massAirFlow;
/** Fuel rate */
@property(nonatomic,  readonly) NSNumber * _Nullable fuelRate;
/** Fuel level in percent */
@property(nonatomic,  readonly) NSNumber * _Nullable fuelLevel;
/** Battery voltage */
@property(nonatomic,  readonly) NSNumber * _Nullable batteryVoltage;
/** Fuel rail pressure */
@property(nonatomic,  readonly) NSNumber * _Nullable fuelRailPressure;
/** Commanded exhaust gas recirculation */
@property(nonatomic,  readonly) NSNumber * _Nullable commandedExhaustGasRecirculation;
/** Barometric pressure */
@property(nonatomic,  readonly) NSNumber * _Nullable barometricPressure;
/** Catalyst temperature */
@property(nonatomic,  readonly) NSNumber * _Nullable catalystTemperature;
/** Absolute load value */
@property(nonatomic,  readonly) NSNumber * _Nullable absoluteLoad;
/** Ambient air temperature */
@property(nonatomic,  readonly) NSNumber * _Nullable ambientAirTemperature;
/** Engine oil temperature */
@property(nonatomic,  readonly) NSNumber * _Nullable engineOilTemperature;
/** Charge air cooler temperature */
@property(nonatomic,  readonly) NSNumber * _Nullable chargeAirCoolerTemperature;
/** Exhaust gas temperature */
@property(nonatomic,  readonly) NSNumber * _Nullable exhaustGasTemperature;

/** The time when the chunk arrived at the smart phone as unix timestamp. */
@property(nonatomic, readonly) NSDate * _Nullable receivingTime;

@end

@protocol TTLAccelerationDatagram <NSObject, NSCopying, NSMutableCopying>
/** Acceleration */
@property(nonatomic, readonly) NSNumber * _Nullable accelerationAvgX;
@property(nonatomic, readonly) NSNumber * _Nullable accelerationAvgY;
@property(nonatomic, readonly) NSNumber * _Nullable accelerationAvgZ;
@property(nonatomic, readonly) NSNumber * _Nullable accelerationMaxX;
@property(nonatomic, readonly) NSNumber * _Nullable accelerationMaxY;
@property(nonatomic, readonly) NSNumber * _Nullable accelerationMaxZ;
@property(nonatomic, readonly) NSNumber * _Nullable accelerationMinX;
@property(nonatomic, readonly) NSNumber * _Nullable accelerationMinY;
@property(nonatomic, readonly) NSNumber * _Nullable accelerationMinZ;

/** The time when the chunk arrived at the smart phone as unix timestamp. */
@property(nonatomic) NSDate * _Nonnull receivingTime;

@end

/** A Representation of a histogram.
*
* The values (events) are aligned to bands that are addressed by numbers starting by 0;
*
* Each band contained in this histogram describes specific value ranges.
* E. g. a histogram of acceleration events is split in bands of 100 milli-g.
*
* The histogram may NOT include values for every band.
*/
@interface TTLDrivingProfileHistogram : NSObject
- (instancetype _Nonnull)initWithValues:(NSDictionary * _Nonnull)values;

- (NSString * _Nonnull)description;

- (BOOL)isEqual:(id _Nullable)other;

- (BOOL)isEqualToHistogram:(TTLDrivingProfileHistogram * _Nullable)histogram;

- (NSUInteger)hash;

/* The number of the highest band in this histogram */
@property NSNumber * _Nullable maxBand;
/* Values arranged by keys of NSNumber representing the band and starting with 0*/
@property(readonly) NSDictionary * _Nonnull valuesByBands;
@end

@protocol TTLDrivingProfileDatagram <NSObject, NSMutableCopying, NSCopying>

@required

/** Begin of the time frame */
@property(readonly) NSDate * _Nullable begin;

/** End of the time frame */
@property(readonly) NSDate * _Nullable end;

/** The time when the chunk arrived at the smart phone as unix timestamp. */
@property(readonly) NSDate * _Nullable receivingTime;

// Scalar values

/** Mileage in meters */
@property(readonly) NSNumber * _Nullable mileage;

/** Accumulated time with RPM > 0 in seconds */
@property(readonly) NSNumber * _Nullable drivingTime;

/** Sum of idling periods in seconds */
@property(readonly) NSNumber * _Nullable idleTime;

/** Maximum speed in km/h */
@property(readonly) NSNumber * _Nullable maxSpeed;

// Histograms
/* Time driven in certain speed ranges */
@property(readonly) TTLDrivingProfileHistogram * _Nullable drivingTimeBySpeed;

/*Distance traveled in certain speed ranges*/
@property(readonly) TTLDrivingProfileHistogram * _Nullable mileageBySpeed;

/*Number of braking events within a g force range*/
@property(readonly) TTLDrivingProfileHistogram * _Nullable braking;

/*Number of speedup events within a g force range*/
@property(readonly) TTLDrivingProfileHistogram * _Nullable speedUp;

/*Number of cornering left events within a g force range*/
@property(readonly) TTLDrivingProfileHistogram * _Nullable corneringLeft;

/*Number of cornering right events within a g force range*/
@property(readonly) TTLDrivingProfileHistogram * _Nullable corneringRight;

/*Engine run time at different rpm ranges*/
@property(readonly) TTLDrivingProfileHistogram * _Nullable engineRpm;

/*Engine run time at different load situations*/
@property(readonly) TTLDrivingProfileHistogram * _Nullable engineLoad;

@end

@protocol TTLServiceODBDataSubscriptionProtocol <NSObject>

- (void)onOBDDatagram:(id <TTLOBDDatagram> _Nonnull)datagram;

@end

@protocol TTLServiceAccelerationDataSubscriptionProtocol <NSObject>

- (void)onAccelerationDatagram:(id <TTLAccelerationDatagram> _Nonnull)datagram;

@end

@protocol TTLServiceDrivingProfileDataSubscriptionProtocol <NSObject>

- (void)onDrivingProfileDatagram:(id <TTLDrivingProfileDatagram> _Nonnull)datagram;

@end
